@@ -12,26 +12,30 @@ class Socket(object):
     def __init__(self, server, sid):
         self.server = server
         self.sid = sid
-        self.queue = getattr(self.server.async['queue'],
-                             self.server.async['queue_class'])()
+        self.queue = self.create_queue()
         self.last_ping = time.time()
         self.connected = False
         self.upgraded = False
+        self.closing = False
         self.closed = False
 
+    def create_queue(self):
+        return getattr(self.server._async['queue'],
+                       self.server._async['queue_class'])()
+
     def poll(self):
         """Wait for packets to send to the client."""
         try:
             packets = [self.queue.get(timeout=self.server.ping_timeout)]
             self.queue.task_done()
-        except self.server.async['queue'].Empty:
+        except self.server._async['queue'].Empty:
             raise IOError()
         if packets == [None]:
             return []
         try:
             packets.append(self.queue.get(block=False))
             self.queue.task_done()
-        except self.server.async['queue'].Empty:
+        except self.server._async['queue'].Empty:
             pass
         return packets
 
@@ -98,23 +102,25 @@ class Socket(object):
 
     def close(self, wait=True, abort=False):
         """Close the socket connection."""
-        self.server._trigger_event('disconnect', self.sid, async=False)
-        if not abort:
-            self.send(packet.Packet(packet.CLOSE))
-        self.closed = True
-        if wait:
-            self.queue.join()
+        if not self.closed and not self.closing:
+            self.closing = True
+            self.server._trigger_event('disconnect', self.sid, async=False)
+            if not abort:
+                self.send(packet.Packet(packet.CLOSE))
+            self.closed = True
+            if wait:
+                self.queue.join()
 
     def _upgrade_websocket(self, environ, start_response):
         """Upgrade the connection from polling to websocket."""
         if self.upgraded:
             raise IOError('Socket has been upgraded already')
-        if self.server.async['websocket'] is None or \
-                self.server.async['websocket_class'] is None:
+        if self.server._async['websocket'] is None or \
+                self.server._async['websocket_class'] is None:
             # the selected async mode does not support websocket
             return self.server._bad_request()
-        websocket_class = getattr(self.server.async['websocket'],
-                                  self.server.async['websocket_class'])
+        websocket_class = getattr(self.server._async['websocket'],
+                                  self.server._async['websocket_class'])
         ws = websocket_class(self._websocket_handler)
         return ws(environ, start_response)
 
@@ -171,6 +177,7 @@ class Socket(object):
             '%s: Upgrade to websocket successful', self.sid)
 
         while True:
+            p = None
             try:
                 p = ws.wait()
             except:
